/* Copyright 1999-2000 Red Hat, Inc.
 *
 * Florian La Roche <Florian.LaRoche@redhat.de>
 *
 * This software may be freely redistributed under the terms of the GNU
 * public license.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "rhutil.h"

#define TMPNAME	".rhtmpXXXXXX"

int
WriteFile (rhconfig_t *c)
{
	char *tmp = NULL, *p;
	int err, fd = -1, unlink_tmp = 0;
	size_t name_len;

	if (BackupFile (c->name))
		goto error;
	name_len = strlen (c->name);
	tmp = rh_xmalloc (name_len + strlen (TMPNAME) + 1);
	strcpy (tmp, c->name);
	if ((p = strrchr (tmp, '/')))
		p++;
	else
		p = tmp;
	strcpy (p, TMPNAME);

	if ((fd = mkstemp(tmp)) < 0)
		goto error;
	if (fchown (fd, c->st.st_uid, c->st.st_gid))
		goto error;
	if (fchmod (fd, c->st.st_mode))
		goto error;
	if (Write (fd, c->buf, c->n))
		goto error;
	(void) close (fd);
	fd = -1;
	unlink_tmp = 1;

	/* XXX should we stat() the file again and see if it is still not
	   changed by some other process? - shouldn't be necessary... */
	if (rename (tmp, c->name))
		goto error;
	free (tmp);
	return 0;

error:
	err = errno;
	if (fd != -1 || unlink_tmp)
		(void) unlink (tmp);
	if (fd != -1)
		(void) close (fd);
	RH_FREE (tmp);
	errno = err;
	return -1;
}
